use std::sync::Arc;

use axum::extract::State;
use maud::{html, Markup, DOCTYPE};

use crate::{clock, data::CombinedFish, AppState};

pub fn layout(content: Markup) -> Markup {
    html! {
        (DOCTYPE)
        html {
            head {
                title { "beacon" }
                meta name="viewport" content="width=device-width";
                link rel="stylesheet" href="/static/style.css";
                script src="/static/htmx.js" {}
            }

            body { (content) }
        }
    }
}

pub fn main_page(state: State<Arc<AppState>>, with_layout: bool) -> Markup {
    let meta = state.data.fish_with_meta();
    let mut values: Vec<&CombinedFish> = meta.values().filter(|f| f.entry.big_fish).collect();
    values.sort_by(|afish, bfish| {
        bfish
            .is_up
            .cmp(&afish.is_up)
            .then(bfish.is_always_up.cmp(&afish.is_always_up))
            .then(bfish.rarity.total_cmp(&afish.rarity).reverse())
            .then(bfish.meta.name_en.cmp(&afish.meta.name_en))
    });
    // values.sort_by(|afish, bfish| {
    //     if !afish.is_up && !bfish.is_up && !afish.windows.is_empty() && !bfish.windows.is_empty() {
    //         bfish
    //             .windows
    //             .first()
    //             .unwrap()
    //             .start_time
    //             .cmp(&afish.windows.first().unwrap().start_time)
    //             .reverse()
    //     } else {
    //         Ordering::Equal
    //     }
    // });
    let template = html! {
        h1 { "Hello! Current ET: " (clock::get_current_eorzea_date().format("%H:%M")) }
        @for fish in values {
            section.up[fish.is_up].alwaysup[fish.is_always_up] {
                .title {
                    h3 { (fish.meta.name_en) }
                    .subtitle {
                        "Patch " (fish.entry.patch)
                    }
                }
                .when {
                    @if let Some(window) = fish.windows.first() {
                        @if fish.is_up || fish.is_always_up {
                            "closes " (window.display_end_time())
                        } @else {
                            "opens " (window.display_start_time())
                        }
                    }
                }
                .how {
                    @for item_id in &fish.entry.best_catch_path {
                        @if let Some(item) = state.data.db_data.items.get(item_id) {
                            span.catchpath title=(item.name_en) {
                                img src=(item.get_icon_url()) width="35";

                                @if let Some(hookset) = item.get_hookset(&state.data) {
                                    img.hookset src=(hookset) width="20";
                                }

                                @if let Some(tug) = item.get_tug(&state.data) {
                                    span.tug { (tug) }
                                }
                            }
                        }
                    }
                }
                .meta {
                    @if fish.entry.start_hour.is_some() && fish.entry.end_hour.is_some() {
                        div {
                            @if !fish.is_always_up {
                                (clock::display_eorzea_time(&clock::set_hm_from_float(&clock::get_current_eorzea_date(), fish.entry.start_hour.unwrap())))
                                " to "
                                (clock::display_eorzea_time(&clock::set_hm_from_float(&clock::get_current_eorzea_date(), fish.entry.end_hour.unwrap())))
                            } @else {
                                "always up!"
                            }
                        }
                    }
                    div { "Rarity: " (format!("{:.2}", fish.rarity * 100.)) "%" }
                }
            }
        }
    };

    if with_layout {
        layout(html! {
            main hx-get="/" hx-trigger="every 10s" {
                (template)
            }
        })
    } else {
        template
    }
}
